/*
  UnLockDA
    00.10.7
*/

#include <PalmOS.h>

#include "Defs.h"

#define DaProgCreatorID  'UNLC'
#define DaProgPrefsID  1

#define DaProgPrefsVersion 0x0001

// preference
typedef struct _prefs {
  Int16 lastX, lastY;
  UInt32 lastUnlock;
} _prefs;
    
// global variable
typedef struct _global {
  _prefs prefs;
  Int16 fromX, fromY;
  Boolean moving;
} _global;

void DAMain();

void start()
{
  DAMain();
}

/* Preference ̏ */
static void InitPrefs(_global *g)
{
  MemSet(&g->prefs, sizeof(_prefs), 0L);
  g->prefs.lastX = DefaultFrameXPos;
  g->prefs.lastY = DefaultFrameYPos;
  g->prefs.lastUnlock = 0;
}

/* vt@Xǂݍ */
static void LoadPrefs(_global *g)
{
  Int16 version;
  UInt16 prefSize;

  version = PrefGetAppPreferences(
    DaProgCreatorID,
    DaProgPrefsID,
    &g->prefs,
    &prefSize,
    true                    // persistent
      );
  if (version == noPreferenceFound) {
   /* Preference Ȃ̂ŁAftH[gݒ */
   InitPrefs(g);
  } else {
    switch (version) {
      default:
        ;
    }
  }
}

/* vt@Xۑ */
static void SavePrefs(_global *g)
{
  PrefSetAppPreferences(
    DaProgCreatorID,
    DaProgPrefsID,
    DaProgPrefsVersion,
    &g->prefs,
    sizeof(g->prefs),
    true                    // persistent
  );
}

/* O[oϐ */
static _global *InitGlobals()
{
  _global *g;
  g = MemPtrNew(sizeof(_global));
  g->moving = false;
  return (g);
}

/* ẽAvP[V擾 ...  DA ł͖gp */
Err GetCurAppName(Char* name)
{
  Err error;
  UInt16 cardNo;
  LocalID dbID;

  error = SysCurAppDatabase(&cardNo, &dbID);
  if(!error)
    error = DmDatabaseInfo(cardNo, dbID, name, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
  return (error);
}

/*  DA ̃^Cg̓Ń^bvꂽ? */
static Boolean IsTitle(RectanglePtr r, Int16 x, Int16 y)
{
  return (0 <= x && x <= r->extent.x - 10 && 0 <= y && y <= 12);
}

/*  DA ̃tH[̓Ń^bvꂽ? */
static Boolean IsInside(RectanglePtr r, Int16 x, Int16 y)
{
  return (0 <= x && x <= r->extent.x && 0 <= y && y <= r->extent.y);
}

/*  DA ̃tH[̊OŃ^bvꂽ? */
static Boolean IsOutside(RectanglePtr r, Int16 x, Int16 y)
{
  return (!IsInside(r, x, y));
}

/* t[ EX-OR [hŕ` */
static void ExorFrame(UInt16 topLeftX, UInt16 topLeftY, _global *g)
{
  RectangleType r;

  r.topLeft.x = topLeftX + 2;
  r.topLeft.y = topLeftY + 2;
  r.extent.x  = FrameXSize  - 4;
  r.extent.y  = FrameYSize - 4;
  WinInvertRectangleFrame(boldRoundFrame, &r);
}

/* Cxgnh[ */
static Boolean DAHandleEvent(EventPtr event, _global *g)
{
  Boolean done = false;
  Boolean handled = false;
  FormPtr form;
  Int16 moveX, moveY;

  switch (event->eType) {
  case appStopEvent:
    /* appStopEvent AƂ̃AvP[Vɑ */
    EvtAddEventToQueue(event);
    done = true;
    handled = true;
    break;
  case keyDownEvent:
    if (event->data.keyDown.modifiers & commandKeyMask) {
      /* HotSync! {^ꂽ? */
      EvtAddEventToQueue(event);
      done = true;
      handled = true;
    }
    break;
    /* ȉ́AhbKu@\̎ */
  case penDownEvent:
    form = FrmGetActiveForm();
    if (IsTitle(&form->window.windowBounds,
      event->screenX, event->screenY)) {
      /* hbOJn */
      g->fromX  = event->screenX;
      g->fromY  = event->screenY;
      g->moving = true;
      FrmEraseForm(form);
      /* hbÓAg\ */
      ExorFrame(0, 0, g);
      handled = true;
    } else if (IsOutside(&form->window.windowBounds,
        event->screenX, event->screenY)) {
      /* tH[̊OŃ^bvꂽ -- I */
      done = true;
      handled = true;
    }
    break;
  case penMoveEvent:
    if (g->moving) {
      form = FrmGetActiveForm();
      moveX = event->screenX - g->fromX;
      moveY = event->screenY - g->fromY;

      /* Oɕ`悵g */
      ExorFrame(0, 0, g);

      /* VtH[ʒuvZ */
      form->window.windowBounds.topLeft.x += moveX;
      form->window.windowBounds.topLeft.y += moveY;
      if (form->window.windowBounds.topLeft.x < FrameWidth) {
        form->window.windowBounds.topLeft.x = FrameWidth;
      }
      if (form->window.windowBounds.topLeft.y < FrameWidth) {
        form->window.windowBounds.topLeft.y = FrameWidth;
      }
      if (form->window.windowBounds.topLeft.x + 
        form->window.windowBounds.extent.x +
        FrameWidth >= form->window.displayWidthV20) {
        form->window.windowBounds.topLeft.x =
        form->window.displayWidthV20 - 
        form->window.windowBounds.extent.x - FrameWidth;
      }
      if (form->window.windowBounds.topLeft.y + 
        form->window.windowBounds.extent.y +
        FrameWidth >= form->window.displayHeightV20) {
        form->window.windowBounds.topLeft.y =
        form->window.displayHeightV20 - 
        form->window.windowBounds.extent.y - FrameWidth;
      }
      /* Vg\ */
      ExorFrame(0, 0, g);
      handled = true;
    }
    break;
  case penUpEvent:
    form = FrmGetActiveForm();
    if (g->moving) {
      /* ȑO̘g */
      ExorFrame(0, 0, g);
      /* vt@XɁAŌ̈ʒuL^ */
      g->prefs.lastX = form->window.windowBounds.topLeft.x;
      g->prefs.lastY = form->window.windowBounds.topLeft.y;
      g->moving = false;
      FrmDrawForm(form);
      handled = true;
    }
    break;
  case ctlSelectEvent:
    if (event->data.ctlEnter.controlID== buttonID_UnLock){
      //  sԂL^ĂB
      g->prefs.lastUnlock = TimGetSeconds();
      SavePrefs(g);
      
      /* UnLockDÅ̐SȂƂBꂾłB */
      PrefSetPreference(prefHidePrivateRecordsV33, false);  // Hide Off
      PwdRemove();  // Remove Password
      PrefSetPreference(prefDeviceLocked, false); // Unlock
      SysReset();   // Reset
      handled = 1;  //  ZbgĂ̂ŕsvƂ͎v܂ꉞ
    }
    break;

  default:
    break;
  }
  if (!handled) {
    FrmHandleEvent(FrmGetActiveForm(), event);
  }
  return (done);
}

/* Cxg[v */
static void EventLoop(_global *g)
{
  EventType event;
  Boolean done = false;
  UInt16 error;
  do {
    EvtGetEvent(&event, evtWaitForever);
    if (SysHandleEvent(&event))
        continue;
    if (MenuHandleEvent(NULL, &event, &error))
        continue;
    done = DAHandleEvent(&event, g);
  } while (!done);
}

/* C֐{ */
void DAMain()
{
  _global *g;
  FormPtr form;
  UInt32 feature = 0;
  char str[32];
  ControlType * labelP;
  
    /* (Fortune DA) DA ̓dNoB*/
  if (FtrGet(DaProgCreatorID, 0, &feature) == 0) {
    /* ł Fortune DA NĂ -- dN */
    return;
  } else {
    /* dN邽߂ɃtB[`[o^ */
    FtrSet(DaProgCreatorID, 0, feature);
  }

  /* O[oϐ̈̊蓖ĂƂ̏ */
  g = InitGlobals();
  if (g == NULL) {
    /* O[oϐ̈̊蓖ĂɎs */
    return;
  }

  /* ȑONꂽƂ Preference [h */
  LoadPrefs(g);
  
  /* CtH[N */
  form  = FrmInitForm(IdDAForm);
  labelP = FrmGetObjectPtr(form, FrmGetObjectIndex(form, LastUnlock_ID)); 
  if(g->prefs.lastUnlock){
    DateTimeType dt; 
    SystemPreferencesType syspref; 
    
    TimSecondsToDateTime (g->prefs.lastUnlock, &dt); 
    PrefGetPreferences(&syspref); 
    DateToAscii(dt.month, dt.day, dt.year,syspref.dateFormat, str); 
    StrCat(str, "  ");
    TimeToAscii(dt.hour, dt.minute, syspref.timeFormat, str + StrLen(str));
    CtlSetLabel(labelP, str);
  }else{
    StrCopy(str, "--/--/--");
    CtlSetLabel(labelP, str);
  }
  
  /* ŌɋNꂽʒu𕜌 */
  form->window.windowBounds.topLeft.x = g->prefs.lastX;
  form->window.windowBounds.topLeft.y = g->prefs.lastY;
  FrmSetActiveForm(form);
  FrmDrawForm(form);

  /* Cxgnh[ɐn */
  EventLoop(g);

  /* AvP[V̏I */
  SavePrefs(g);
  FrmEraseForm(form);
  FrmDeleteForm(form);

  /* tB[`[ */
  if (FtrGet(DaProgCreatorID, 0, &feature) == 0) {
    FtrUnregister(DaProgCreatorID, 0);
  }

  /* O[oϐ̈ */
  MemPtrFree(g);
}
